import glob
import os
from .theme import Theme
from typing import Union

def load_theme(theme_name: str):
    """
    Sets the chosen style and color palette globally.

    :param theme_name: name of the theme to load
    :return: the specified Theme
    :raise ValueError: if a theme is not found
    """
    themes = _get_themes()
    if theme_name in themes.keys():
        return Theme.from_file(themes[theme_name])
    else:
        raise ValueError(f"No theme named '{theme_name}' found.")


def list_themes():
    """
    Returns a list of available theme names.

    :return: a list of available theme names
    """
    return list(_get_themes().keys())


def _get_themes():
    """
    Returns available themes from the theme directory

    :return: a {name: path} dict of all available themes
    """
    loc = os.path.dirname(os.path.abspath(__file__))
    return dict(
        map(
            lambda x: (os.path.basename(x).split(".")[0], x),
            glob.glob(os.path.join(loc, "themes", "*.json")),
        )
    )

def _sample_plot(theme: Union[Theme, str], save_as: str = None):
    """
    Generates a sample plot for a given theme. Optionally saves it to the specified location.

    :param theme: Theme instance or theme name to load.
    :param save_as: path to save the generated plot to
    """
    import seaborn as sns
    import matplotlib.pyplot as plt

    if type(theme) == str:
        theme = load_theme(theme)

    with theme:
        geysers = (
            sns.load_dataset("geyser")
            .rename(
                columns={
                    "duration": "Duration",
                    "kind": "Kind",
                    "waiting": "Waiting",
                }
            )
            .replace({"long": "Long", "short": "Short"})
        )
        fig, ax = plt.subplots(1, 3, figsize=(15, 5))
        # Hacky patched boxplot since seaborn overrides color options otherwise
        # sns.boxplot(x="Kind", y="Duration", data=tips, ax=ax[0])
        ax[0].boxplot(
            [
                geysers.loc[geysers["Kind"] == "Long", "Duration"].tolist(),
                geysers.loc[geysers["Kind"] == "Short", "Duration"].tolist(),
            ],
            vert=True,
            patch_artist=True,
            labels=geysers["Kind"].unique(),
        )
        sns.kdeplot(
            x="Waiting", y="Duration", hue="Kind", data=geysers, ax=ax[1], fill=True
        )
        sns.lineplot(x="Waiting", y="Duration", hue="Kind", data=geysers, ax=ax[2])
        plt.suptitle("Geysers")

        if save_as is not None:
            fig.savefig(
                save_as,
                dpi=75,
                transparent=False,
                facecolor=fig.get_facecolor(),
                bbox_inches="tight",
            )

def make_samples():
    """
    Generates sample plots for all themes to be used in documentation
    """
    for theme in list_themes():
        _sample_plot(theme, f"assets/{theme}.png")




import matplotlib.pyplot as plt
import numpy as np


def rotate_ylabel(degrees: int):
    """
    Rotates the y-labels of the current plot.

    :param degrees: rotation in degrees
    """
    axes = plt.gcf().axes
    for ax_i in axes:
        ax_i.tick_params(axis="y", rotation=degrees)


def rotate_xlabel(degrees: int):
    """
    Rotates the x-labels of the current plot.

    :param degrees: rotation in degrees
    """
    axes = plt.gcf().axes
    for ax_i in axes:
        ax_i.tick_params(axis="x", rotation=degrees)


def offset(distance: int):
    """
    Offsets the plot spines.
    Code partly taken from https://github.com/mwaskom/seaborn/blob/563e96d3be1eaee8db8dfbccf7eed1f1c66dfd31/seaborn/utils.py#L292

    :param distance: offset distance int pt.
    """
    axes = plt.gcf().axes
    for ax_i in axes:
        for side in ["top", "right", "left", "bottom"]:
            ax_i.spines[side].set_position(("outward", distance))


def trim(axis: str):
    """
    Trims axes of a plot to first and last major tick.
    Code partly taken from https://github.com/mwaskom/seaborn/blob/563e96d3be1eaee8db8dfbccf7eed1f1c66dfd31/seaborn/utils.py#L292

    :param axis: axes to apply the trim to. Can be {"x", "y", "both"}.
    """
    # Apply trim to all axes
    for ax_i in plt.gcf().axes:
        if axis in ["x", "both"]:
            # Trim x direction (bottom and top)
            xticks_major = np.asarray(ax_i.get_xticks(minor=False))
            xticks = np.asarray(ax_i.get_xticks(minor=True))
            if xticks.size:
                # Get first and last major ticks
                firsttick = np.compress(
                    xticks_major >= min(ax_i.get_xlim()), xticks_major
                )[0]
                lasttick = np.compress(
                    xticks_major <= max(ax_i.get_xlim()), xticks_major
                )[-1]
                # Trim spines to tick range
                ax_i.spines["bottom"].set_bounds(firsttick, lasttick)
                ax_i.spines["top"].set_bounds(firsttick, lasttick)
                # Update tick values for both minor and major ticks
                xticks = xticks.compress(xticks <= lasttick)
                xticks = xticks.compress(xticks >= firsttick)
                ax_i.set_xticks(xticks, minor=True)
        if axis in ["y", "both"]:
            # Trim y direction (left and right)
            yticks_major = np.asarray(ax_i.get_yticks(minor=False))
            yticks = np.asarray(ax_i.get_yticks(minor=True))
            if yticks.size:
                # Get first and last major ticks
                firsttick = np.compress(
                    yticks_major >= min(ax_i.get_ylim()), yticks_major
                )[0]
                lasttick = np.compress(
                    yticks_major <= max(ax_i.get_ylim()), yticks_major
                )[-1]
                # Trim spines to tick range
                ax_i.spines["left"].set_bounds(firsttick, lasttick)
                ax_i.spines["right"].set_bounds(firsttick, lasttick)
                # Update tick values
                newticks = yticks.compress(yticks <= lasttick)
                newticks = newticks.compress(newticks >= firsttick)
                ax_i.set_yticks(newticks, minor=True)
